/*
 *   Copyright (c) 2006 David Parsons. All rights reserved.
 *   
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *  3. All advertising materials mentioning features or use of this
 *     software must display the following acknowledgement:
 *     
 *   This product includes software developed by David Parsons
 *   (orc@pell.chi.il.us)
 *
 *  4. My name may not be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *     
 *  THIS SOFTWARE IS PROVIDED BY DAVID PARSONS ``AS IS'' AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL DAVID
 *  PARSONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  The licence and distribution terms for any publically available
 *  version or derivative of this code cannot be changed.  i.e. this
 *  code cannot simply be copied and put under another distribution
 *  licence [including the GNU Public Licence.]
 */
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#include "atomdoc.h"

extern "C" say(char*,...);


atomdoc::atomdoc()
{
    commentok = 0;
    author = 0;
    title = 0;
    docfile = 0;
    maptext = 0;
    mapsize = 0;
}


atomdoc::~atomdoc()
{
    if (maptext)
	munmap(maptext, mapsize);
    if (docfile)
	fclose(docfile);
}


int
atomdoc::read(FILE *f)
{
    register c;
    struct stat sz;

    if (docfile) {
	say("A document has already been loaded");
	return 0;
    }

    if ( (docfile = tmpfile()) == 0 ) {
	say("Cannot create tempfile: %s", strerror(errno));
	return 0;
    }

    fprintf(docfile, "<entry xmlns=\"http://www.w3.org/2005/Atom\">\n");
    fprintf(docfile, "<title>%s</title>\n", title);
    fprintf(docfile, "<author>%s</author>\n", author);
    fprintf(docfile, "<control>\n"
		     "<comments>%s</comments>\n"
		     "<text-format>html</text-format>\n"
		     "</control>\n", commentok ? "yes" : "no");
    fprintf(docfile, "<content><![CDATA[\n");

    while ( (c =getc(f)) != EOF ) {
	if (c == '[' || c == ']')
	    fprintf(docfile, "&#%d;", c);
	else
	    fputc(c, docfile);
    }

    fprintf(docfile, "]]>\n");
    fprintf(docfile, "</content>\n");
    fprintf(docfile, "</entry>\n");
    fflush(docfile);

    if (fstat(fileno(docfile), &sz) == -1) {
	say("cannot stat tempfile");
	fclose(docfile);
	return 0;
    }
    maptext = mmap(0, sz.st_size, PROT_READ,MAP_SHARED,fileno(docfile),0L);
    mapsize = sz.st_size;
    return 1;
} /* read(FILE*) */



int
atomdoc::read(char *f)
{
    FILE *input = fopen(f, "r");

    if (input) {
	int status = read(input);
	fclose(input);
	return status;
    }
    say("can't read %s", f);
    return 0;
}
