/*
 *   Copyright (c) 2006 David Parsons. All rights reserved.
 *   
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *  3. All advertising materials mentioning features or use of this
 *     software must display the following acknowledgement:
 *     
 *   This product includes software developed by David Parsons
 *   (orc@pell.chi.il.us)
 *
 *  4. My name may not be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *     
 *  THIS SOFTWARE IS PROVIDED BY DAVID PARSONS ``AS IS'' AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL DAVID
 *  PARSONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  The licence and distribution terms for any publically available
 *  version or derivative of this code cannot be changed.  i.e. this
 *  code cannot simply be copied and put under another distribution
 *  licence [including the GNU Public Licence.]
 */

extern "C" {
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
}

#include "www.h"
#include "atomdoc.h"


/*#define XMLPOST	"/post/what2.cgi"*/
#define XMLPOST	"/post/xmlpost.cgi"


char *pgm;

static void
usage(int rc)
{
    fprintf(stderr, "usage: %s [-cdv] [-U user] [-P passwd] "
			      "weblog title [article]\n", pgm);
    exit(rc);
}


extern "C" void say(char *fmt,...)
{
    int written;
    va_list ptr;

    fprintf(stderr, "%s: ", pgm);
    va_start(ptr, fmt);
    written = vfprintf(stderr, fmt, ptr);
    va_end(ptr);
    putc('\n', stderr);
}

extern "C" main(int argc, char **argv)
{

    extern char *basename(char*);
    int opt;
    char *title = 0;
    char *user = 0;
    char *passwd = 0;
    class atomdoc document;
    int c_flag = 0,
	d_flag = 0,
	v_flag = 0;

    pgm = basename(argv[0]);

    opterr = 1;
    while ( (opt = getopt(argc, argv, "cdvU:P:T:?")) != EOF)
	switch (opt) {
	case 'T':
	    title = optarg;
	    break;
	case 'P':
	    passwd = optarg;
	    break;
	case 'U':
	    user = optarg;
	    break;
	case 'c':
	    c_flag = 1;
	    break;
	case 'd':
	    d_flag = 1;
	    break;
	case 'v':
	    v_flag = 1;
	    break;
	case '?':
	    usage(0);
	default:
	    usage(1);
	}

    argc -= optind;
    argv += optind;

    if (argc < 2)
	usage(1);


    if (user == 0) {
	char *p = (char*)strchr(argv[0], '~');

	if (p) {
	    user = strdup(1+p);

	    if (p = (char*)strchr(user, '/'))
		*p = 0;
	}
	else {
	    say("Can't figure out user name from %s", argv[0]);
	    exit(1);
	}
    }

    if (passwd == 0) {
	char prompt[strlen(user)+strlen("'s password")+3];

	sprintf(prompt, "%s's password: ", user);

	if ( (passwd = getpass(prompt)) == 0 || strlen(passwd) == 0 )
	    exit(0);
    }
    document.set_author(user);
    document.set_title(argv[1]);
    if (c_flag)
	document.allow_comments();

    if ( (argc < 3) || (strcmp(argv[2], "-") == 0) ) {
	if ( !document.read(stdin) ) {
	    say("can't read document from standard input");
	    exit(1);
	}
    }
    else if ( !document.read(argv[2]) ) {
	say("can't read document from %s", argv[1]);
	exit(1);
    }

    if (!document.ok())
	exit(1);

    char url[strlen(argv[0])+strlen(XMLPOST)];
    sprintf(url, "%s" XMLPOST, argv[0]);

    class WWW web(url);

    if (v_flag)
	web.setverbose();
    if (d_flag)
	web.setdebug();

    if (web.ok()) {
	web.set_auth(user,passwd, WWW::A_BASIC);
	web.set_content_type("application/atom+xml");
	web.post(document.data(), document.size());
	int status = web.reply(0,0);
	exit( (status == 201) ? 0 : 1);
    }
    exit(1);
} /* atompost */
