/*
 *   Copyright (c) 2004-2006 David Parsons. All rights reserved.
 *   
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *  3. All advertising materials mentioning features or use of this
 *     software must display the following acknowledgement:
 *     
 *   This product includes software developed by David Parsons
 *   (orc@pell.chi.il.us)
 *
 *  4. My name may not be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *     
 *  THIS SOFTWARE IS PROVIDED BY DAVID PARSONS ``AS IS'' AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL DAVID
 *  PARSONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  The licence and distribution terms for any publically available
 *  version or derivative of this code cannot be changed.  i.e. this
 *  code cannot simply be copied and put under another distribution
 *  licence [including the GNU Public Licence.]
 */
extern "C" {
#include <stdio.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>

#include <malloc.h>
#include <string.h>
#include <stdarg.h>
#include <sys/utsname.h>

extern void say(char*,...);
extern char *to64(char*);

}

#include "www.h"

WWW::WWW(char *name):linebufferedinput()
{
    struct hostent *hostent;
    struct servent *p;
    struct sockaddr_in sock;
    char *c;

    flags = IS_EOF;
    fd = -1;

    if (strncmp(name, "http://", 7) != 0) {
	say("Unsupported URL scheme");
	return;
    }

    /* copy the name into scratch space so we can split into
     * http://{host}/{path}
     */
    if ( (host = new char[strlen(name+7)+1]) == 0 ) {
	say("Out of memory connecting to %s", name);
	return;
    }
    strcpy(host, name+7);

    /* split off the path
     */
    if (c = (char*)strchr(host, '/')) {
	*c++ = 0;
	path = c;
    }
    else
	path = "";


    /* find the port to connect to
     */
    if ( (c = (char*)strchr(host, ':')) ) {
	*c++ = 0;
	port = atoi(c);
    }
    else
	port = 80;


    /* then find the host
     */
    if ( (hostent = gethostbyname(host)) == 0) {
	struct in_addr addr;
	struct hostent hbuf;
	char *alist[1];

	if ( (addr.s_addr = inet_addr(host)) == -1) {
	    say("unknown host %s.", host);
	    return;
	}
	hbuf.h_name = host;
	hbuf.h_addr_list = alist;
	hbuf.h_addr = (char*)&addr;
	hbuf.h_length = sizeof(addr);
	hbuf.h_addrtype = AF_INET;
	hbuf.h_aliases = 0;
	hostent = &hbuf;
    }

    /* open up a socket, then return a WWW* attached to the socket
     */
    sock.sin_family = hostent->h_addrtype;
    sock.sin_port = htons(port);
    memcpy(&sock.sin_addr, hostent->h_addr, hostent->h_length);

    if ( (fd = socket(hostent->h_addrtype, SOCK_STREAM, 0)) < 0)
	say("www socket open failed: %s", strerror(errno));
    else if (connect(fd, (struct sockaddr *)&sock, sizeof(sock)) < 0) {
	say("www socket open failed: %s", strerror(errno));
	close(fd);
	fd = -1;
    }
    else {
	flags &= ~IS_EOF;
	setfd(fd);
    }
}


WWW::~WWW()
{
    if (fd != -1) close(fd);
}


void
WWW::set_content_type(char *typ)
{
    if (typ == 0)
	return;

    if ( content_type = new char[strlen(typ)+1] ) {
	strcpy(content_type, typ);
    }
    else {
	say("out of memory allocating content_type");
	flags |= IS_EOF;
    }
}


void
WWW::set_auth(char *user, char *pass, int auth_type)
{
    char *plaintext, *enc;

    if (auth_type != A_BASIC) {
	say("BASIC auth only!");
	flags |= IS_EOF;
	return;
    }
    else if ( plaintext = new char[strlen(user) + strlen(pass) + 2] ) {
	sprintf(plaintext, "%s:%s", user, pass);
	if ( (enc = to64(plaintext)) && (auth = new char[strlen(enc)+1]) ) {
	    strcpy(auth, enc);
	    return;
	}
    }
    say("out of memory allocating auth string");
    flags |= IS_EOF;
}


void
WWW::post(char *content, int content_length)
{
    struct iovec vec[20];
    int nvec = 0;
    struct utsname buf;

#define addv(a)	( vec[nvec  ].iov_base = (a), \
		  vec[nvec++].iov_len = strlen(a) )

    uname(&buf);

    addv("POST /");
    addv(path);
    addv(" HTTP/1.1\r\nHost: ");
    addv(buf.nodename);

    if (content_type) {
	addv("\r\nContent-Type: ");
	addv(content_type);
    }

    char content_length_string[20];

    sprintf(content_length_string, "%d", content_length);

    addv("\r\nContent-Length: ");

    addv(content_length_string);

    if (auth) {
	addv("\r\nAuthorization: Basic ");
	addv(auth);
    }
    addv("\r\n\r\n");
    vec[nvec].iov_base = content;
    vec[nvec].iov_len  = content_length;

    writev(fd, vec, nvec+1);

    if (verbose()) {
	int start=1, i, len;
	char *p;

	for (i=0; i<=nvec; i++) {
	    p = (char*)vec[i].iov_base;
	    len = vec[i].iov_len;
	    while (len-- > 0) {
		if (start)
		    fwrite(">> ", 3, 1, stdout);
		start = (*p == '\n');
		putchar(*p++);
	    }
	}
	if (!start) putchar('\n');
    }
}


int
WWW::reply(char **bfr, int *size)
{
    char *text;
    int firstline = 1;
    int headers = 0;
    char version[40];

    if ( (text = readline()) == 0 )
	return 500;

    if (sscanf(text, "%40s %d", version, &status) == 2)
	headers = (strncasecmp(version, "HTTP/1.", 7) == 0);

    if (debug())
	printf("<< %s\n", text);

    while ( text = readline() ) {
	if (strlen(text) == 0)
	    headers=0;
	if (debug())
	    printf("<< %s\n", text);
    }

    return status;
}


char *
linebufferedinput::readline()
{
    char *p, *ret;
    int sz;

    if (iobfr == 0)
	iobfr = (char*)malloc(iolen = 200);

    if (iobfr == 0) {
	say("out of memory allocating line buffer");
	return 0;
    }

    if (ioptr > 0) {
	iosize -= ioptr;
	if (iosize > 0)
	    memmove(iobfr, iobfr+ioptr, iosize);
    }

    while (1) {
	if ( iosize && (p = (char*)memchr(iobfr, '\n', iosize)) ) {
	    ret = iobfr;

	    if ( (p > iobfr) && (p[-1] == '\r') )
		p[-1] = 0;
	    p[0] = 0;
	    ioptr = 1+(p-iobfr);

	    return ret;
	}
	if ( iosize == iolen )
	    if ( (iobfr = (char*)realloc(iobfr, iolen *= 2)) == 0 ) {
		say("out of memory expanding line buffer");
		return 0;
	    }

	sz = read(fd, iobfr+iosize, iolen-iosize);

	if ( (sz == -1) && (errno != EINTR) ) {
	    say("error %s reading input", strerror(errno));
	    return 0;
	}

	if (sz == 0) {
	    ret = iosize ? iobfr : 0;
	    if (ret)
		iobfr[iosize] = 0;
	    ioptr = 0;
	    iosize = 0;
	    return ret;
	}
	iosize += sz;
    }
}


#ifdef DEBUG
extern "C" void say(char *fmt,...)
{
    int written;
    va_list ptr;

    va_start(ptr, fmt);
    written = vfprintf(stderr, fmt, ptr);
    va_end(ptr);
    putc('\n', stderr);
}

main()
{
    int status;
    class WWW foo("http://weblog.pell.portland.or.us/~orc/post/what.cgi");

    foo.setverbose();
    foo.setdebug();

    if (foo.ok()) {
	foo.set_auth("orc", "here", WWW::A_BASIC);
	foo.set_content_type("application/text");

	foo.post("HERE WE GO!\r\n", 13);

	status = foo.reply(0,0);

	printf("status is %d\n", status);
    }
}
#endif
