/*
 *   Copyright (c) 2004 David Parsons. All rights reserved.
 *   
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *  3. All advertising materials mentioning features or use of this
 *     software must display the following acknowledgement:
 *     
 *   This product includes software developed by David Parsons
 *   (orc@pell.chi.il.us)
 *
 *  4. My name may not be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *     
 *  THIS SOFTWARE IS PROVIDED BY DAVID PARSONS ``AS IS'' AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL DAVID
 *  PARSONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  The licence and distribution terms for any publically available
 *  version or derivative of this code cannot be changed.  i.e. this
 *  code cannot simply be copied and put under another distribution
 *  licence [including the GNU Public Licence.]
 */

#ifndef __WWW_H
#define __WWW_H

extern "C" {
#include <stdlib.h>
}

class linebufferedinput {
private:
    int   fd;
    char *iobfr;	/* allocated buffer */
    int   iolen;	/* how big is it? */
    int   iosize;	/* how much is used? */
    int   ioptr;	/* how far in is start of line? */
public:
    inline  linebufferedinput() { iobfr = 0; iolen=0; ioptr = 0; fd=-1; }
    inline ~linebufferedinput() { if (iobfr) free(iobfr); }

    char *readline();
    inline int setfd(int f) { fd = f; }
};


/* Magic www connection structure
 */

class WWW:public linebufferedinput {
public:
    enum { IS_DEBUG=0x01, IS_VERBOSE=0x02, IS_EOF=0x04 };
    enum { A_BASIC=1, A_DIGEST=2 };

    WWW(char*);
    ~WWW();

    inline int  debug()		{ return flags & IS_DEBUG; }
    inline int  verbose()	{ return flags & IS_VERBOSE; }
    inline int  ok()		{ return !(flags & IS_EOF); }
    inline void setdebug()	{ flags |= IS_DEBUG; }
    inline void setverbose()	{ flags |= IS_VERBOSE; }

    void set_content_type(char *);
    void set_auth(char *, char *, int authtype);
    int status;

    void  post(char *, int);
    void  get();
    void  head();
    int   reply(char **, int *);

private:
    char *host;			/* the host we connect to */
    int   port;			/* at this port */
    char *path;			/* the path name there */
    int   fd;			/* file descriptor connecting us to the host */
    char* content_type;		/* user-supplied content-type for post */
    char* auth;			/* auth information, "encrypted" */
    int   flags;		/* status flags */
} ;


/* conversation is:
 *
 *  foo = new WWW("http://host/path");
 *
 *  if (foo.ok()) {
 *      foo.set_content_type("content/type");
 *      foo.set_content(text, size of text);
 *      foo.set_auth("user", "passwd", WWW:A_BASIC);
 *      foo.post();
 *	return foo.reply(0,0);
 *  }
 */
#endif/*__WWW_H*/
